declare module 'route-mapper' {

  type SINGLETON_ACTION = 'index' | 'create' | 'new' | 'show'  | 'update' | 'destroy' | 'edit';
  type ACTION = 'show' | 'create' | 'update' | 'destroy'  | 'edit';

  interface ISingletonResourceOpts {
    as?: string;
    camelCase?: boolean;
    controller?: string;
    except?: ACTION[];
    only?: ACTION[];
    plural?: boolean;
  }
  
  interface IResourceOpts {
    as?: string;
    camelCase?: boolean;
    controller?: string;
    except?: SINGLETON_ACTION[];
    only?: SINGLETON_ACTION[];
    param?: string;
    path?: string;
  }

  interface IRouteMapperOptions {
    pathNames?: {
      new?: string
      edit?: string
    }
  }
  
  interface IParseArgs<T> {
    arg: string | T | Function;
  }
  
  interface IRouteArgs {
    to?: string;
   }
  
  class Resource {
    public readonly defaultActions: ACTION[];
    public readonly actions: ACTION[];
    public readonly plural: string;
    public readonly singular: string;
    public readonly memberName: string;
    public readonly collectionName: string;
    public readonly resourceScope: { controller: string };
    public readonly collectionScope: string;
    public readonly memberScope: string;
    public readonly nestedParam: string;
    public readonly nestedScope: string;
    public newScope(newPath: string): string;
  }

  class SingletonResource extends Resource {
    constructor(entity: string, options: ISingletonResourceOpts);
  }
  
  class Route {
    private _action: string;
    private _controller: string;

    constructor($scope: any, path: string, options: IRouteArgs);
  }

  class RouteMapper {
    get(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    head(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    patch(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    post(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    put(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    delete(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;

    acl(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    bind(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    checkout(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    connect(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    copy(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    link(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    lock(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    'm-search'(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    merge(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    mkactivity(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    mkcalendar(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    mkcol(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    move(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    notify(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    options(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    propfind(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    proppatch(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    purge(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    rebind(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    report(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    search(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    subscribe(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    trace(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    unbind(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    unlink(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    unlock(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    unsubscribe(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    
    match(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    controller(controller: string, options?: IRouteArgs, callback?: Function): this;
    root(path: string, options?: any): this;
    resource(path, opts?: ISingletonResourceOpts): this;
    resources(path, opts?: IResourceOpts): this;
    new(cb: () => any): this;
    nested(cb: () => any): this;
    namespace(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    concern(name: string, callable?: () => any, cb?: () => any): this;
    concerns(path?: string | IRouteArgs, options?: IRouteArgs | Function, callback?: Function): this;
    scope(opts: any, cb: Function): this;
    applyCommonBehaviorFor(method: string, resources: Resource[], options: any, cb: () => any): this;
    resourceScope(kind: string, resource: Resource, cb: () => any): this;
    constructor(options?: IRouteMapperOptions);
    
    prefixNameForAction(as: string, action: string);
    parentResource: any;
    $scope: any;
    helpers: any;
  }
  
  export { RouteMapper as default };
}